# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
import BZtypeList
import BZcommon
import BZsceneWriter
import Blender
import sys
import BZRegistry
class reader:
	def __init__(self):
		# retrieve BZPrefs to avoid trying to load a file that isn't there
		if 'linux' in sys.platform:
			BZRegistry.LoadConfigData('BZFlag')
		# end mod retrieve
		self.debug=2
		self.groupList={}
		self.instList={}
		self.groupInst={}
		self.renderedGroupInstances={}
		self.inGroup=False
		self.importedNames=[]
		self.index=0
		self.opt=''
		self.indent=''
		self.skipDefine=0
		self.lineNo=0
		self.matlist=[]
		self.telelist=[]
		self.telelist.append(0)
		self.changedNames={}
		self.changedGroups={}
		try:
			material=Blender.Material.Get('BZtexface')
		except NameError:
			material=Blender.Material.New('BZtexface')
			material.setMode('TexFace')
			material.setSpec(0.1)
		self.matlist.append('BZtexface')
	#enddef
	def read(self,filename):
		inobject=False
		inface=False
		meshtext=0
		self.pipe=open(filename,'r')
		for line in self.pipe.readlines():
			self.lineNo+=1
			keywd=str(retList(line)[0])
			if keywd:
				key1=keywd[0].lower()
				keywd=key1+keywd[1:]
			datalst=retList(line)[1:]
			datastr=' '.join(datalst)
			if keywd!='' and self.debug>3:print '%s:%s' % (self.lineNo,line)
			if keywd=='matref' and not meshtext:						#numbered materials
				if datastr in self.changedNames:
					datastr=self.changedNames[datastr]					#recover material name changes
				try:
					Blender.Material.Get(datastr)
				except NameError:
					try:
						q=int(datastr)
						matname=str(self.matlist[q])
						Blender.Material.Get(matname)
						datastr=matname
					except:
						print 'Unknown Material %s' % datastr
						keywd='#'
			########################################################################
			######################## Import groups start proper ###########################
			
			if keywd=='': pass			#skip whitespace
			elif keywd[0]=='#':	pass	#It's a comment, skip it
			
			# ##################################################
			elif self.inGroup and keywd=='enddef':
				if self.debug >2: print 'group objects are %s' % self.groupList[self.currentGroup.name]
				module=BZsceneWriter.define
				writer=module.importer(self.currentGroup)
				writer.run()
				if self.debug:print '\033[1;32mCreated group definition placeholder:-\033[0m %s' % (self.currentGroup.name)
				self.inGroup=False
				setattr(BZtypeList.define,'objects',[])
				self.indent=''
			
			# ##################################################
			elif not inobject: #check keyword to start a valid object #
				#try:
					obj=BZtypeList.Objects[keywd]()
					inobject=True
					if self.debug:print '%s\033[1;34mStarting import of object type:-\033[0m %s' % (self.indent, obj.type())
					if obj.type()=='define':
						inobject=self.startGroup(retList(line))
					if obj.type()=='options':
						self.startoptions()
					if obj.type()=='teleporter':obj.telenum=self.teleStart(retList(line))
					if obj.type()=='group':
						parName=self.checkGroupName(datastr)
						if self.groupInst.has_key(parName): self.groupInst[parName]+=1
						else: self.groupInst[parName]=1
						obj.parent=parName
						obj.name='I%s#%s' % (obj.parent[1:],self.groupInst[parName])
						self.index+=1
				#except:
				#	if self.debug:print 'Unknown object type %s :Skipping' % keywd
				#	pass
				#endtry
			
			# ###############################################
			elif inface:
				if keywd in face.strings:
					setattr(face, keywd,datastr)
				elif keywd in face.attribs:
					setattr(face, keywd,datalst)
				elif keywd.lower() in face.bools:
					setattr(face, keywd.lower(),True)
				elif keywd=='endface':
					obj.face.append(face)
					inface=False
					if self.debug>2:print '\033[1;32mFinished importing face:-\033[0m'
				else:
					if self.debug:print '\033[1;33mUnknown attribute \033[0m%s\033[1;33m for Object:-\033[0m face' % (keywd)				
			
			# ################################################
			elif inobject:
				if keywd=='from':keywd='ffrom'## cheap python reserved word workaround
				if keywd=='diffuse':keywd='color'## cheap colour alias workaround, some poeple need to learn to spell!!
				if keywd=='type':keywd='ttype'##w-weapon type alias
				if obj.type()=='dynamicColor' and keywd!='name' and keywd!='end':		#change keyword for dyncol 
					keywd1=str(retList(line)[0])
					keywd2=str(retList(line)[1])
					keywd='_'.join([keywd1,keywd2])
					datalst=retList(line)[2:]
					datastr=' '.join(datalst)
				if obj.type()=='options':
					self.opt.write(line)
				if meshtext:                  		#store drawType options for imported meshes
					self.mtex.write(line)
					if keywd in ['lod','matref']:
						meshtext+=1
					elif keywd=='end':
						meshtext-=1					#end
				# ##########
				elif keywd=='face':					#start instance of class face
					face=BZtypeList.face()
					inface=True						#end
				# ##########
				elif keywd=='drawInfo':				#start to store drawType options for imported meshes
					meshtext=1
					if obj.name==BZtypeList.Objects[obj.type()]().name:	#check has proper name
						obj.name='%s%s' % (obj.name,self.index)
						self.index+=1
					try:
						self.importedNames.index(obj.name)				#check has unique name
						obj.name='%s%s' % (obj.name,self.index)
						self.index+=1
					except:
						pass
					self.mtex=Blender.Text.New(obj.name)
					self.mtex.write('drawInfo\n')	#ready to store text
				elif keywd=='name' and obj.type()=='group':				#yeah, skip changing the name *again*
					pass
				# ###########      proper import section   #############
				elif keywd in obj.strings:
					setattr(obj,keywd,datastr)
				
				elif keywd in obj.attribs:
					setattr(obj,keywd,datalst)
				
				elif keywd in obj.params:
					lis=getattr(obj,keywd,[])
					app=tuple(datalst)
					lis.append(app)
					setattr(obj,keywd,lis)
				
				elif keywd.lower() in obj.bools:
					setattr(obj,keywd.lower(),True)
				
				elif keywd in obj.materials:
					datastr=datastr[datastr.find(' ')+1:]   					#remove matref
					if datastr in self.changedNames:
						datastr=self.changedNames[datastr]					#recover material name changes
					try:
						Blender.Material.Get(datastr)
						setattr(obj,keywd,datastr)
					except NameError:
						try:
							q=int(datastr)
							matname=str(self.matlist[q])
							Blender.Material.Get(matname)
							datastr=matname
							setattr(obj,keywd,datastr)
						except:
							print '\033[1;31mError: On map line %s Unknown Material %s' % (self.line,datastr)
				
				elif keywd=='end':
					inobject=False
					if obj.type()=='material':												#must register material name changes
						if len(obj.name)>15:												#make name short enough to be used in blender
							newname='mat%s%s' %(obj.name[:12],self.index)
							self.index+=1
							self.changedNames[obj.name]=newname								#store the change
							obj.name=newname
						elif obj.name==BZtypeList.Objects[obj.type()]().name:				#check not std
							newname='%s%s' % (obj.name,self.index)
							self.index+=1
							self.changedNames[obj.name]=newname								#store the change
							obj.name=newname
						try:
							Blender.Object.Get(obj.name)									#check unique in blender
							newname='%s%s' % (obj.name,self.index)
							self.index+=1
							self.changedNames[obj.name]=newname								#store the change
							obj.name=newname						
						except:
							pass
						try:
							self.importedNames.index(obj.name)								#check unique
							newname='%s%s' % (obj.name,self.index)
							self.index+=1
							self.changedNames[obj.name]=newname								#store the change
							obj.name=newname
						except:
							self.importedNames.append(obj.name)								#add to list
					elif obj.type()=='group':
						#self.instList[obj.name]=obj.parent
						if self.debug>3: print 'Group instance named %s with parent %s' % (obj.name,obj.parent)
						pass
					else:
						telename=0
						if obj.type()=='teleporter' and obj.name=='Tele': 					#Workaround for bad tele syntax (grr, can't people read man pages) 
							self.telelist.pop()
							telename=1
							
						if len(obj.name)>15:												#make name short enough to be used in blender
							obj.name=obj.name[:15]
						if obj.name==BZtypeList.Objects[obj.type()]().name:					#check not std
							obj.name='%s%s' % (obj.name,self.index)
							self.index+=1
						try:
							Blender.Object.Get(obj.name)									#check unique in blender
							newname='%s%s' % (obj.name,self.index)
							self.index+=1
							self.changedNames[obj.name]=newname								#store the change
							obj.name=newname						
						except:
							pass
						try:
							self.importedNames.index(obj.name)								#check unique
							obj.name='%s%s' % (obj.name,self.index)
							self.index+=1							
						except:
							self.importedNames.append(obj.name)								#add to list
							
						if telename:									 					#Workaround for bad tele syntax (grr, can't people read man pages) 
							self.telelist.append(obj.name)
						
					if self.inGroup:
						obj.ingroup=self.currentGroup.name
						print self.currentGroup.name
						self.groupList[self.currentGroup.name].append(obj.name)		#append to glist
					try:
						module=BZsceneWriter.Object[obj.type()]						#do the import
						writer=module.importer(obj)
						writer.run()
					except KeyError:
						pass 														#special object types must not be registered.
					if obj.type()=='material':
						self.matlist.append(obj.name)								#register material
					for param in obj.params:
						setattr(BZtypeList.Objects[obj.type()],param,[])			#reset class instance, Fuck knows why?? it's an instance ffs
					if self.debug:print '%s\033[1;32mFinished importing %s object:-\033[0m %s' % (self.indent,obj.type(),obj.name)
				else:
					if self.debug and obj.type()!='options':print '\033[1;33mUnknown attribute \033[0m%s\033[1;33m for Object:-\033[0m %s' % (keywd,obj.type())
				#endif
			else:
				pass
			#endif
		#endfor
		self.pipe.close()
		self.finishGroups()
		self.finishInstances()
		Blender.Window.EditMode(0)
		Blender.Window.EditMode(1)
		Blender.Window.EditMode(0)
		Blender.Scene.GetCurrent().update()
		Blender.Redraw()
		if self.debug:print '\033[1;32m**** Finished importing map %s ****\033[0m' % filename
	#enddef
	
	def finishGroups(self):
		for groupName in self.groupList:
			if self.debug:print '\033[1;34mFinish up group definition:-\033[0m %s' % groupName
			group=Blender.Object.Get(groupName)
			children=[]
			for childName in self.groupList[groupName]:
				child=Blender.Object.Get(childName)
				children.append(child)
				child.layers=[3]
			group.makeParent(children)
		Blender.Redraw()
	#enddef
	
	def finishInstances(self):
		Blender.Window.ViewLayers([1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20])
		self.instList=[]
		#for instName in self.instList: #iterate for know group instances
		scene=Blender.Scene.GetCurrent()
		for inst in scene.objects:
			if BZcommon.propertyCheck(inst,'parent'):
				self.renderInstance(inst.getName())
		Blender.Window.ViewLayers([1,2])
		Blender.Redraw()	
	
	def renderInstance(self,instName):
		if self.debug:print '\033[1;34mFinishing up rendering group instance:-\033[0m %s' % instName
		for objs in Blender.Object.GetSelected(): #unselect all
			objs.sel=0
		children=[]
		inst=Blender.Object.Get(instName)					#get the instance placeholder
		groupName=BZcommon.propertyCheck(inst,'parent')		#get the placeholder name
		group=Blender.Object.Get(groupName)					#get the group placeholder

		if groupName in self.changedGroups:					#recover changed group names
			groupName=self.changedGroups[groupName]		

		place=inst.getLocation()							#store spacial params
		size=inst.getSize()
		spin=inst.getEuler()
		inst.setLocation([0,0,0])
		inst.setSize([1,1,1])
		inst.setEuler(Blender.Mathutils.Euler(0,0,0))		#store spacial params
		for objname in self.groupList[groupName]:         	#select to be duplicated from dictionary
			obj=Blender.Object.Get(objname)					#get original
			obj.sel=1										#make selected
		Blender.Object.Duplicate(mesh=1)					#Duplicate all
		obj=Blender.Object.GetSelected()					#Pick up the new objects
		Blender.Redraw()					
		children.append(obj)								#append to list for parenting ??
		for possInst in Blender.Object.GetSelected():		#setup loop for checking into children bieng groups instances  
			possInst.layers=[2]
			if 		BZcommon.propertyCheck(possInst,'ingroup') \
					and BZcommon.propertyCheck(possInst,'bzflag')=='group':	# is 'ingroup' and is a 'group'	
				self.renderInstance(possInst.getName())			#cyclic recall
		inst.makeParent(obj)
		inst.setSize(size)
		inst.setLocation(place)	
		inst.setEuler(spin)
		Blender.Redraw()

	
	def startoptions(self):
		self.opt=Blender.Text.New('prefix')
		self.opt.write('options\n')
		return
	def teleStart(self,start):
		name='Tele'
		try:
			name=start[1]
		except:
			print '\033[1;31mError: On map line %s, teleporter with no name!\033[0m' % self.lineNo
			#fatalError()# set the telename check instead
		try:
			self.importedNames.index(name)
			print '\033[1;31mError: On map line %s, two teleporters with same name!\033[0m' % self.lineNo
			fataError()
		except:
			pass
		self.telelist.append(name)
		telenum=self.telelist.index(name)
		return telenum
	#enddef
	
	def checkGroupName(self,name):
		if self.changedGroups.has_key(name):
			newname=self.changedGroups[name]
		else:												#make name short enough to be used in blender
			newname='G%s%s' %(name[:8],self.index)
			self.index+=1
			self.changedGroups[name]=newname							#store the change
			name=newname
		return newname
	#enddef
	
	
	def startGroup(self,start):
		if self.inGroup:
			print '\033[1;31mError: On map line %s, nested group definition!\033[0m' % self.lineNo
			fatalError()
		name=' '.join(start[1:])
		name=self.checkGroupName(name)
		#if self.groupList.has_key(name):
		#	print '\033[1;31mError: On map line %s, group definition allready exists!\033[0m' % self.lineNo
		#	fatalError()
		if not self.groupList.has_key(name): self.groupList[name]=[] #make entry if not exists
		self.inGroup=True
		self.currentGroup=BZtypeList.define()
		self.currentGroup.name=name
		self.indent='%s  ' % self.indent
		return False
	#enddef			
#endclass
def retList(line):
	retlist=[]
	line=line.replace('\t',' ')
	#line=line.replace(',',' ')  not needed??
	while line.find('  ')>=0:
		line=line.replace('  ',' ')
	line=line.strip()
	line=line + ' '
	while line!='':
		retlist.append(line[:line.find(' ')])
		line=line[line.find(' ')+1:]
	return retlist
#enddef
def fatalError():
	print '\033[1;31m#####################################################'
	print '#######  Exited on fatal map file error!  ###########'
	print '#####################################################\033[0m'
#enddef
