# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import BZcommon
import Blender
import math
from BZcommon import retList,propertyCheck,rad2deg,rnd
try:
	ro=Blender.Registry.GetKey('BZFlag', True)['ro']
except:
	ro=3
class bzprim:

	def updateto(self): #update properties to scene
		BZcommon.setProperties(self.obj)
		#name
		obj=Blender.Object.Get(self.obj.name)
		#position
		x,y,z=self.obj.position
		obj.setLocation(float(x),float(y),float(z))
		#size
		x,y,z=self.obj.size
		obj.setSize(float(x),float(y),float(z))
		#zrotation
		z=BZcommon.deg2rad(float(self.obj.rotation[0]))
		obj.setEuler((0.0,0.0,z))

	def updatefrom(self):
		pass

class bzmesh:

	def updateto(self): #update properties to scene
		BZcommon.setProperties(self.obj)
		obj=Blender.Object.Get(self.obj.name)
		#scale
		if 'scale' in self.obj.attribs:
			if self.obj.scale:
				c=obj.getSize()
				d=[]
				for i in 0,1,2:
					d.append(c[i]*float(self.obj.scale[i]))
				obj.setSize(float(d[0]),float(d[1]),float(d[2]))
			self.obj.scale=None
		#shift
		if self.obj.shift:
			a=self.obj.shift
			b=obj.getLocation()
			c=[]
			for i in 0,1,2:
				c.append(float(b[i])+float(a[i]))
			obj.setLocation(c)
			self.obj.shift=None
		#spin
		if self.obj.spin:
			nm=BZcommon.spin2matrix(self.obj.spin)
			trfmat=Blender.Mathutils.Matrix(nm[0],nm[1],nm[2])
			oldEU=obj.getEuler()
			for a in 0,1,2:
				oldEU[a]=BZcommon.rad2deg(oldEU[a])
			oldmat=oldEU.toMatrix()
			newmat=oldmat*trfmat
			newEU=newmat.toEuler()
			for a in 0,1,2:
				newEU[a]=BZcommon.deg2rad(newEU[a])
				newEU[a]=-newEU[a]
				if newEU[a]<0:
					n=(math.pi*2)-abs(newEU[a])
			obj.setEuler(newEU)
			self.obj.spin=None
		def updatefrom(self):
			pass

class BlObject:

	def updateFrom(self):
		for prop in self.obj.getAllProperties():
			if prop.name in self.out.attribs:
				if propertyCheck(self.obj,prop.name):
					data=self.cleanStr(propertyCheck(self.obj,prop.name))
					setattr(self.out,prop.name,data)
			elif prop.name in self.out.bools:
				if propertyCheck(self.obj,prop.name):
					setattr(self.out,prop.name,True)
			elif prop.name in self.out.materials:
				if propertyCheck(self.obj,prop.name):
					data=self.cleanStr(propertyCheck(self.obj,prop.name))
					setattr(self.out,prop.name,data)
			elif prop.name in self.out.strings:
				if propertyCheck(self.obj,prop.name):
					data=self.cleanStr(propertyCheck(self.obj,prop.name))
					setattr(self.out,prop.name,data)
			self.out.name=self.obj.name

	def cleanStr(self,string):
		string=str(string)
		string=string.replace('\'','')
		string=string.replace('[','')
		string=string.replace(']','')
		string=string.replace(',','')
		return string

	def writeStrings(self):
		for attrib in self.out.strings:
			data=getattr(self.out,attrib,None)
			if attrib in ['name','parent','telenum','ingroup']:
				pass
			elif data:
				self.pipe.write('\t%s %s\n' %(attrib,data))
		self.pipe.flush()

	def writeAttribs(self,ExceptList=('position','size','rotation','shift','scale','spin')):
		for attrib in self.out.attribs:
			data=getattr(self.out,attrib,None)
			if attrib not in ExceptList and data:
				data=self.cleanStr(data)
				self.pipe.write('\t%s %s\n' %(attrib,data))
		self.pipe.flush()

	def writeMaterials(self):
		for attrib in self.out.materials:
			data=getattr(self.out,attrib,None)
			if data:
				if data not in ['BZtexface']:#unwanted materials
					self.pipe.write('\tmatref %s %s\n' %(attrib,data))
		self.pipe.flush()

	def writeBools(self):
		for attrib in self.out.bools:
			data=getattr(self.out,attrib,None)
			if data:
				self.pipe.write('\t%s\n' %(attrib))
		self.pipe.flush()

	def writePrim(self):
		x,y,z=self.obj.getLocation()
		th=self.obj.getEuler()[2]
		th=BZcommon.rad2deg(th)
		p,q,r=self.obj.getSize()
		if x or y or z 	and 'position' 	in self.out.attribs: 		self.pipe.write('\tposition %s %s %s\n' %(	rnd(x,ro),
																											rnd(y,ro),
																											rnd(z,ro)))	
		if th 			and 'rotation' 	in self.out.attribs: 		self.pipe.write('\trotation %s\n' 		%	rnd(th,ro))
		if p or q or r 	and 'size' 		in self.out.attribs: 		self.pipe.write('\tsize %s %s %s\n' 	%(	rnd(p,ro),
																											rnd(q,ro),
																											rnd(r,ro)))
		self.pipe.flush()

	def writeMesh(self,zoffset=0):# TODO add scale transform
		#spin
		euler=self.obj.getEuler()
		x,y,z=euler.unique()						#remove unneeded 360deg rotations
		if x or y or z:
			eu=[rad2deg(-x),rad2deg(-y),rad2deg(z)]	#convert to degrees
			for a in 0,1,2:							#absolute
				if eu[a]<0: eu[a]=360+eu[a]			#absolute
			Eu=Blender.Mathutils.Euler(eu)			#create euler object
			Qu=Eu.toQuat()							#create Quaternion object
			a=Qu.angle								#extract angle
			p,q,r=Qu.axis							#extract axis
			self.pipe.write('\tspin %s %s %s %s\n' %(	rnd(a,ro),
														rnd(p,ro),
														rnd(q,ro),
														rnd(r,ro)
													))	
		#apply position stored in mesh data
		if 'position' in self.out.attribs:
			data=self.cleanStr(self.out.position)
			datalst=retList(data)
			if float(datalst[0]) or float(datalst[1]) or float(datalst[2]):
				self.pipe.write('\tposition %s\n' %data)	

		#apply rotation stored in mesh data

		if 'rotation' in self.out.attribs:
			data=self.cleanStr(self.out.rotation)
			if float(data):
				self.pipe.write('\trotation %s\n' %data)

		#apply size stored in mesh data
		if 'size' in self.out.attribs:
			data=self.cleanStr(self.out.size)
			datalst=retList(data)
			if float(datalst[0]) or float(datalst[1]) or float(datalst[2]):
				self.pipe.write('\tsize %s\n' %data)


		#scale
		p,q,r=self.obj.getSize()
		if p!=1.0 or q!=1.0  or r!=1.0 	and 'scale' 	in self.out.attribs: 		self.pipe.write('\tscale %s %s %s\n' 	%(	rnd(p,ro),
																										rnd(q,ro),
																										rnd(r,ro)))		

		#shift
		x,y,z=self.obj.getLocation()
		if x or y or z 	and 'shift'	in self.out.attribs: 		self.pipe.write('\tshift %s %s %s\n' %(	rnd(x,ro),
																										rnd(y,ro),
																										rnd(z,ro)))


