# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import Blender
from Blender import NMesh, Image, Object
import BZcommon
import BZtypeList
import math
from math import sin, cos 
from BZobject import bzmesh,BlObject
import BZtypeList


class importer(bzmesh):

	def __init__(self,bzobj):
		BZPrefs=Blender.Registry.GetKey('BZFlag', True)
		try:
			self.wallimage=Image.Get('boxwall.png')
		except NameError:
			self.wallimage=Blender.Image.Load(BZPrefs['bztexpath']+'boxwall.png')
		try:
			self.topimage=Blender.Image.Get('roof.png')
		except NameError:
			self.topimage=Blender.Image.Load(BZPrefs['bztexpath']+'roof.png')
		self.obj=bzobj
		self.matlib={}

	def run(self):
		obj=NMesh.New(self.obj.name)
		obj.addMaterial(Blender.Material.Get('BZtexface'))
		i=1
		if self.obj.matref:
			obj.addMaterial(Blender.Material.Get(self.obj.matref))
			i=2
		for mat in self.obj.materials:
			cmat=getattr(self.obj,mat,None)
			if cmat:
				try:
					obj.addMaterial(Blender.Material.Get(cmat))	
					self.matlib[mat]=i
					i+=1
				except AttributeError:
					j=0
					for psmat in obj.getMaterials(-1):
						if psmat.name==cmat:self.matlib[mat]=j
						j+=1
		if not self.obj.flatshading:
			obj.mode=32
		obj.hasFaceUV(1)
		a=float(BZcommon.deg2rad(self.obj.angle[0]))
		d=int(self.obj.divisions[0])
		da=float(a/d)
		r=1.0-float(self.obj.ratio[0])
		vertlist=[]
		facelist=[(0,1,3,2)]
		fac=math.sqrt(2)
		mx,my,mz=self.obj.size
		mx=float(mx)
		my=float(my)
		mz=float(mz)
		dx,dy,dz=self.obj.position
		dx=float(dx)
		dy=float(dy)
		dz=float(dz)
		for sect in range(0,d+1):
			ca=float(sect*da)+0.25*math.pi#+BZcommon.deg2rad(float(self.obj.rotation))
			ix=fac*r*cos(ca)*mx+dx
			iy=fac*r*sin(ca)*my+dy
			ox=fac*cos(ca)*mx+dx
			oy=fac*sin(ca)*my+dy
			vertlist.append((ix,iy,1*mz+dz))
			vertlist.append((ix,iy,0+dz))
			vertlist.append((ox,oy,1*mz+dz))
			vertlist.append((ox,oy,0+dz))
		for verts in vertlist:
			v=NMesh.Vert(verts[0],verts[1],verts[2])
			obj.verts.append(v)			

		face=(0,1,3,2)
		f=NMesh.Face()		
		for verts in face:	f.v.append(obj.verts[verts])
		if self.matlib.has_key('startside'):#startface
			f.mat=self.matlib['startside']
		elif	self.obj.matref:
			f.mat=1
		else:
			f.mat=0
			f.image=self.wallimage
			f.flag=1
			f.mode=5
		f.uv=[(0.0,1.0),(0.0,0.0),(1.0,0.0),(1.0,1.0)]		
		obj.faces.append(f)		
		facelist=[]
		for sect in range (0,d):#inside faces
			o=4*sect
			facelist.append((o+4,o+5,o+1,o+0))
		for face in facelist:
			f=NMesh.Face()
			for verts in face:
				f.v.append(obj.verts[verts])
			if self.matlib.has_key('inside'):
				f.mat=self.matlib['inside']
			elif	self.obj.matref:
				f.mat=1
			else:
				f.mat=0
				f.image=self.wallimage
				f.flag=1
				f.mode=5
				f.uv=[(0.0,1.0),(0.0,0.0),(1.0,0.0),(1.0,1.0)]		
			obj.faces.append(f)	
		facelist=[]
		for sect in range (0,d):#outside faces
			o=4*sect
			facelist.append((o+2,o+3,o+7,o+6))
		for face in facelist:
			f=NMesh.Face()
			for verts in face:
				f.v.append(obj.verts[verts])
			if self.matlib.has_key('outside'):
				f.mat=self.matlib['outside']
			elif	self.obj.matref:
				f.mat=1
			else:
				f.mat=0
				f.image=self.wallimage
				f.flag=1
				f.mode=5
			f.uv=[(0.0,1.0),(0.0,0.0),(1.0,0.0),(1.0,1.0)]		
			obj.faces.append(f)				
		facelist=[]
		face=((o+6,o+7,o+5,o+4))
		f=NMesh.Face()
		for verts in face:	f.v.append(obj.verts[verts])
		if self.matlib.has_key('endside'):#endface
			f.mat=self.matlib['endside']
		elif	self.obj.matref:
			f.mat=1
		else:
			f.mat=0
			f.image=self.wallimage
			f.flag=1
			f.mode=5
		f.uv=[(0.0,1.0),(0.0,0.0),(1.0,0.0),(1.0,1.0)]		
		obj.faces.append(f)		
		facelist=[]
		for sect in range (0,d):#top faces
			o=4*sect
			facelist.append((o+0,o+2,o+6,o+4))#topface
		for face in facelist:
			f=NMesh.Face()
			for verts in face:
				f.v.append(obj.verts[verts])
			if self.matlib.has_key('top'):
				f.mat=self.matlib['top']
			elif	self.obj.matref:
				f.mat=1
			else:
				f.mat=0
				f.image=self.topimage
				f.flag=1
				f.mode=5
			f.uv=[(0.0,1.0),(0.0,0.0),(1.0,0.0),(1.0,1.0)]		
			obj.faces.append(f)	
		facelist=[]
		for sect in range (0,d):#bottom
			o=4*sect
			facelist.append((o+5,o+7,o+3,o+1))#bottomface
		for face in facelist:
			f=NMesh.Face()
			for verts in face:
				f.v.append(obj.verts[verts])
			if self.matlib.has_key('bottom'):
				f.mat=self.matlib['bottom']
			elif	self.obj.matref:
				f.mat=1
			else:
				f.mat=0
				f.image=self.topimage
				f.flag=1
				f.mode=5
			f.uv=[(0.0,1.0),(0.0,0.0),(1.0,0.0),(1.0,1.0)]		
			obj.faces.append(f)	
		scobj=Object.New('Mesh',self.obj.name)
		scobj.link(obj)
		scene = Blender.Scene.GetCurrent ()
		scene.objects.link (scobj)
		scobj.setEuler((0,0,BZcommon.deg2rad(float(self.obj.rotation[0]))))
		self.updateto()	
		Blender.Redraw()


class export(BlObject):
	def __init__(self,blobj,pipe):
		self.pipe=pipe
		self.out=BZtypeList.meshbox()
		self.obj=blobj
	
	def run(self):
		BZPrefs=Blender.Registry.GetKey('BZFlag', True)
		ro=BZPrefs['ro']
		self.updateFrom()
		self.pipe.write('meshbox\n')
		self.pipe.write('\tname %s\n' % self.out.name)
		self.writeMaterials()
		self.writeBools()
		self.writeStrings()
		self.writeAttribs()
		self.writeMesh()
		self.pipe.write('end\n\n')
		self.pipe.flush()
		
		

