# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****


import Blender
from Blender import NMesh, Image, Object
import BZcommon
from BZobject import bzprim,BlObject
import BZtypeList


class importer(bzprim):

	def __init__(self,bzobj):
		BZPrefs=Blender.Registry.GetKey('BZFlag', True)
		try:
			image=Image.Get('pyrwall.png')
		except NameError:
			image=Image.Load(BZPrefs['bztexpath']+'pyrwall.png')
		self.obj=bzobj
		#name=self.obj.name[0]
		#self.obj.name=name
		self.image=image
	
	def run(self):
		obj=NMesh.New(self.obj.name)
		obj.addMaterial(Blender.Material.Get('BZtexface'))
		obj.hasFaceUV(1)
		if self.obj.flipz:
			self.obj.verts=(	[1.0,1.0,1.0],
							[1.0,-1.0,1.0],
							[-1.0,-1.0,1.0],
							[-1.0,1.0,1.0],
							[0.0,0.0,0.0]
							)
			self.obj.faces=(	[3,2,1,0], 	#base
			[0,1,4],	#+X
			[2,3,4],	#-X
			[0,4,3],	#+Y
			[1,2,4],	#-Y
			)
		for verts in self.obj.verts:
			v=NMesh.Vert(verts[0],verts[1],verts[2])
			obj.verts.append(v)
		for face in self.obj.faces:
			f=NMesh.Face()
			for verts in face:
				f.v.append(obj.verts[verts])
			obj.faces.append(f)
		for f in obj.faces:
			f.flag=1
			f.mode=5
			f.mat=0
			f.image=self.image
			f.uv=[(0.0,1.0),(0.0,0.0),(1.0,0.0),(1.0,1.0)]
		scobj=Object.New('Mesh',self.obj.name)
		scobj.link(obj)
		scene = Blender.Scene.GetCurrent ()
		scene.objects.link (scobj)
		self.updateto()
		Blender.Redraw()



class export(BlObject):
	def __init__(self,blobj,pipe):
		self.pipe=pipe
		self.out=BZtypeList.pyramid()
		self.obj=blobj
	
	def run(self):
		BZPrefs=Blender.Registry.GetKey('BZFlag', True)
		ro=BZPrefs['ro']
		self.updateFrom()
		self.pipe.write('pyramid\n')
		self.pipe.write('\tname %s\n' % self.out.name)
		self.writeBools()
		self.writeStrings()
		self.writeAttribs()
		self.writePrim()
		self.pipe.write('end\n\n')
		self.pipe.flush()
		
		
